import { __awaiter } from 'tslib';
import { ɵɵdefineInjectable, Injectable, EventEmitter, Component, Input, Output, NgModule } from '@angular/core';
import { getBackend } from '@tensorflow/tfjs';
import { load } from '@tensorflow-models/blazeface';
import { load as load$1 } from '@tensorflow-models/mobilenet';
import { Subject } from 'rxjs';

class LoaderService {
    constructor() {
        this.counter = 0;
        this.loaderState = new Subject();
    }
    show() {
        this.counter++;
        this.displayLoader();
    }
    hide() {
        this.counter--;
        this.displayLoader();
    }
    displayLoader() {
        this.loaderState.next(this.counter);
    }
    get verifyStateLoader() {
        return this.loaderState.asObservable();
    }
}
LoaderService.ɵprov = ɵɵdefineInjectable({ factory: function LoaderService_Factory() { return new LoaderService(); }, token: LoaderService, providedIn: "root" });
LoaderService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
LoaderService.ctorParameters = () => [];

class PsmFaceCaptchaComponent {
    constructor(loaderService) {
        this.loaderService = loaderService;
        this.onerror = new EventEmitter();
        this.onloader = new EventEmitter();
        this.ondata = new EventEmitter();
    }
    ngOnInit() {
        this.getLoaderState();
        getBackend();
        this.loadBlazeFace();
        this.loadMobilenet();
    }
    validateInputData() {
    }
    loadBlazeFace() {
        return __awaiter(this, void 0, void 0, function* () {
            this.loaderService.show();
            this.blazefaceModel = yield load().then(res => res).catch(() => null);
            this.loaderService.hide();
        });
    }
    loadMobilenet() {
        return __awaiter(this, void 0, void 0, function* () {
            this.loaderService.show();
            this.mobilenetModel = yield load$1().then(res => res).catch(() => null);
            this.loaderService.hide();
        });
    }
    getPrediction(canvas) {
        return __awaiter(this, void 0, void 0, function* () {
            this.loaderService.show();
            try {
                const face = yield this.blazefaceModel.estimateFaces(canvas, false);
                const data = yield this.mobilenetModel.classify(canvas);
                console.log(face);
                console.log(data);
            }
            catch (error) {
                alert('error');
            }
            this.loaderService.hide();
        });
    }
    sendError() {
    }
    getLoaderState() {
        this.loaderService.verifyStateLoader.subscribe(res => {
            if (res)
                this.onloader.emit(true);
            else
                this.onloader.emit(false);
        });
    }
}
PsmFaceCaptchaComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-psm-face-captcha',
                template: "<lib-camera (onprediction)=\"getPrediction($event)\"></lib-camera>\r\n",
                styles: [":host{box-sizing:border-box;display:block;height:100%;left:0;position:fixed;top:0;width:100%}"]
            },] }
];
PsmFaceCaptchaComponent.ctorParameters = () => [
    { type: LoaderService }
];
PsmFaceCaptchaComponent.propDecorators = {
    imgCamera: [{ type: Input }],
    imgSwitch: [{ type: Input }],
    cameraAspect: [{ type: Input }],
    instruction: [{ type: Input }],
    onerror: [{ type: Output }],
    onloader: [{ type: Output }],
    ondata: [{ type: Output }]
};

class CameraComponent {
    constructor(loaderService) {
        this.loaderService = loaderService;
        this.constraints = {
            video: {
                width: { min: 640, ideal: 1920, max: 1920 },
                height: { min: 480, ideal: 1080, max: 1080 },
                facingMode: 'user'
            },
            audio: false
        };
        this.onprediction = new EventEmitter();
        this.onerror = new EventEmitter();
    }
    ngOnInit() {
        this.video = document.getElementById('video');
    }
    ngAfterViewInit() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log(this.constraints);
            yield this.getUserMedia(this.constraints);
        });
    }
    initCanvas() {
        this.video.width = this.video.getBoundingClientRect().width;
        this.video.height = this.video.getBoundingClientRect().height;
        const canvas = document.getElementById('canvas');
        canvas.width = 200;
        canvas.height = this.video.height * 200 / this.video.width;
    }
    takepicture() {
        const canvas = document.getElementById('canvas');
        canvas.getContext('2d').drawImage(this.video, 0, 0, canvas.width, canvas.height);
        this.photo = canvas.toDataURL('image/png');
        this.onprediction.emit(canvas);
        this.video.pause();
        this.video.srcObject = null;
        this.stream.getTracks().forEach(track => track.stop());
    }
    switchCamera() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.constraints.video.facingMode === 'user')
                this.constraints.video.facingMode = 'environment';
            else
                this.constraints.video.facingMode = 'user';
            yield this.getUserMedia(this.constraints);
        });
    }
    getUserMedia(constraints) {
        return __awaiter(this, void 0, void 0, function* () {
            this.loaderService.show();
            if (this.stream) {
                this.video.pause();
                this.video.srcObject = null;
                this.stream.getTracks().forEach(track => track.stop());
            }
            const stream = yield navigator.mediaDevices.getUserMedia(constraints).then((stream) => stream).catch(error => error);
            if (stream) {
                if (stream === 'DOMException: Permission denied') {
                    alert('error');
                }
                else {
                    this.stream = stream;
                    this.video.srcObject = stream;
                    this.initCanvas();
                    this.video.play();
                }
            }
            this.loaderService.hide();
        });
    }
    ngOnDestroy() {
        if (this.stream)
            this.stream.getTracks().forEach(track => track.stop());
    }
}
CameraComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-camera',
                template: "\r\n<video id=\"video\" playsinline [muted]=\"'muted'\"></video>\r\n<button (click)=\"takepicture()\" style=\"position: fixed; bottom: 5%; left: 25%;\">Take picture</button>\r\n<canvas id=\"canvas\"></canvas>\r\n<!--<div style=\"position: fixed; top: 0; height: 100px; background-color: aqua;\" (click)=\"facingMode === 'user' ? getRearCamera() : getFrontalCamera()\">Cambio</div>-->\r\n<div style=\"position: fixed; top: 0; height: 100px; background-color: aqua;\" (click)=\"switchCamera()\">Cambio</div>\r\n",
                styles: [":host,:host video{display:block;height:100%;width:100%}:host video{object-fit:cover}:host canvas{display:none}:host .container-photo{bottom:5%;height:15%;position:fixed;right:5%;width:20%}:host .container-photo .photo{max-height:100%;max-width:100%}"]
            },] }
];
CameraComponent.ctorParameters = () => [
    { type: LoaderService }
];
CameraComponent.propDecorators = {
    onprediction: [{ type: Output }],
    onerror: [{ type: Output }]
};

class PsmFaceCaptchaModule {
}
PsmFaceCaptchaModule.decorators = [
    { type: NgModule, args: [{
                declarations: [PsmFaceCaptchaComponent, CameraComponent],
                imports: [],
                exports: [PsmFaceCaptchaComponent]
            },] }
];

/*
 * Public API Surface of psm-face-captcha
 */

/**
 * Generated bundle index. Do not edit.
 */

export { PsmFaceCaptchaComponent, PsmFaceCaptchaModule, LoaderService as ɵa, CameraComponent as ɵb };
//# sourceMappingURL=psm-face-captcha.js.map
